unit UDyMik;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  { Unit containing TLabel }
  StdCtrls,
  { Broker Units }
  mfunstr, trpcb, xwbut1;

type
  { Actual linked list holder of the record }
  PointList = ^PointRec;
  PointRec = record
    Identifier : String;
    RecIndex   : Integer;
    ObjectItem : TObject;
    Next       : PointList;
  end;

  { TDynamicList
    The DynamicList object is not a new idea.  It was patterned after the TList
    object and the TStrings object.  I wanted to develop an object that can
    handle items in a standard Delphi way and make it easier for me to program. }
  TDynamicList = class(TObject)
  private
    FCount    : Integer;
    FTopItem  : PointList;
    FLastItem : PointList;
    function GetItem(Index : Integer) : TObject; Virtual;
  public
    constructor Create; Virtual;
    destructor Destroy; Override;
    procedure Add(Value : TObject); Virtual;
    procedure Clear; Virtual;
    procedure SetIdentifier(Value : String; Index : Integer); Virtual;
    function Find(Identifier : String) : Integer; Virtual;
    property Item[Index : Integer] : TObject read GetItem;
    property Count : Integer read FCount;
  end;

  TDynamicLabel = class(TComponent)
  private
    { Private declarations }
    FRPCBroker       : TRPCBroker;
    FList            : TDynamicList;
    FRemoteProcedure : String;
    FRemoteTag       : String;
    FEvents          : TStringList;
    procedure MakeLabels(var MasterList : TStringList); Virtual;
    procedure SeparateLabel(var MasterList, SingleList : TStringList); Virtual;
    procedure CreateLabel(SingleLabel : TStringList); Virtual;
    procedure LoadEvent(CurrEvent : String; Curr : TLabel);
    procedure LoadParams(Curr : TLabel);
    procedure ChangeParam(SingleLabel : TStringList; ListIndex : Integer);
    Function StrToBool(Value : String) : Boolean;
    Function BoolToStr(Value : Boolean) : String;
    procedure SetAlign(Nu : TLabel; Info : String);
    procedure SetAlignment(Nu : TLabel; Info : String);
    procedure SetAutoSize(Nu : TLabel; Info : String);
    procedure SetCaption(Nu : TLabel; Info : String);
    procedure SetColor(Nu : TLabel; Info : String);
    procedure SetCursor(Nu : TLabel; Info : String);
    procedure SetDragCursor(Nu : TLabel; Info : String);
    procedure SetDragMode(Nu : TLabel; Info : String);
    procedure SetEnabled(Nu : TLabel; Info : String);
    procedure SetFontColor(Nu : TLabel; Info : String);
    procedure SetHeight(Nu : TLabel; Info : String);
    procedure SetLeft(Nu : TLabel; Info : String);
    procedure SetName(Nu : TLabel; Info : String);
    procedure SetParentColor(Nu : TLabel; Info : String);
    procedure SetParentFont(Nu : TLabel; Info : String);
    procedure SetParentShowHint(Nu : TLabel; Info : String);
    procedure SetShowAccelChar(Nu : TLabel; Info : String);
    procedure SetShowHint(Nu : TLabel; Info : String);
    procedure SetTag(Nu : TLabel; Info : String);
    procedure SetTop(Nu : TLabel; Info : String);
    procedure SetTransparent(Nu : TLabel; Info : String);
    procedure SetVisible(Nu : TLabel; Info : String);
    procedure SetWidth(Nu : TLabel; Info : String);
    procedure SetWordWrap(Nu : TLabel; Info : String);
    procedure SetOnClick(Nu : TLabel; Info : String);
    procedure SetOnDblClick(Nu : TLabel; Info : String);
    procedure SetOnDragDrop(Nu : TLabel; Info : String);
    procedure SetOnDragOver(Nu : TLabel; Info : String);
    procedure SetOnEndDrag(Nu : TLabel; Info : String);
    procedure SetOnMouseDown(Nu : TLabel; Info : String);
    procedure SetOnMouseMove(Nu : TLabel; Info : String);
    procedure SetOnMouseUp(Nu : TLabel; Info : String);
    procedure SetOnStartDrag(Nu : TLabel; Info : String);
    procedure Click(Sender : TObject);
    procedure DblClick(Sender : TObject);
    procedure DragDrop(Sender,Source : TObject; X,Y : Integer);
    procedure DragOver(Sender,Source : TObject; X,Y : Integer;
                       State : TDragState; var Accept : Boolean);
    procedure EndDrag(Sender,Target : TObject; X,Y : Integer);
    procedure MouseDown(Sender : TObject; Button : TMouseButton;
                        Shift : TShiftState; X,Y : Integer);
    procedure MouseMove(Sender : TObject; Shift : TShiftState; X,Y : Integer);
    procedure MouseUp(Sender : TObject; Button : TMouseButton;
                      Shift : TShiftState; X,Y : Integer);
    procedure StartDrag(Sender : TObject; var DragObject : TDragObject);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); Override;
    destructor Destroy; Override;
    procedure DeActivate; Virtual;
    procedure Activate; Virtual;
  published
    { Published declarations }
    property RPCBroker : TRPCBroker read FRPCBroker write FRPCBroker;
    property RemoteProcedure : String read FRemoteProcedure write FRemoteProcedure;
    property RemoteTag : String read FRemoteTag write FRemoteTag;
  end;

procedure Register;

implementation

constructor TDynamicList.Create;
{ Constructor : Create
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Initialize the TDynamicList object.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   FCount := 0;
   FTopItem := Nil;
   FLastItem := Nil;
end;

destructor TDynamicList.Destroy;
{ Destructor  : Destroy
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Ensure that the objects are freed and that the linked list is
                destroyed.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Clear;
   inherited Destroy;
end;

function TDynamicList.Find(Identifier : String) : Integer;
{ Function    : Find
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Given an identifier find the index if possible.  Case is
                unimportant.  If not found, then -1 will be returned.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Pointer used to help traverse the linked list }
   Curr : PointList;
begin
   { Initialize }
   Result := -1;
   Curr := FTopItem;
   Identifier := UpperCase(Identifier);
   { Traverse until found or run out of linked list }
   while((Result = -1) and (Curr <> Nil)) do
   begin
      if (Curr^.Identifier = Identifier) then
         Result := Curr^.RecIndex;
      Curr := Curr^.Next;
   end;
end;

procedure TDynamicList.SetIdentifier(Value : String; Index : Integer);
{ Procedure   : SetIdentifier
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Given an identifier and an index, set the appropriate field.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Pointer used to help traverse the linked list }
   Curr : PointList;
begin
   { Initialize }
   Curr := FTopItem;
   Value := UpperCase(Value);
   { Traverse until found or run out of linked list }
   while(Curr <> Nil) do
   begin
      if (Curr^.RecIndex = Index) then
         Curr^.Identifier := Value;
      Curr := Curr^.Next;
   end;
end;

function TDynamicList.GetItem(Index : Integer) : TObject;
{ Function    : GetItem
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Given an index, find the object in the list located at that
                index and return it.  If the index is out of range then Nil is
                returned.  Later this should be modified to raise an error.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Pointer used to help traverse the linked list }
   Curr : PointList;
begin
   { Initialize }
   Result := Nil;
   Curr := FTopItem;
   { Traverse until found or run out of linked list }
   while((Result = Nil) and (Curr <> Nil)) do
   begin
      if (Curr^.RecIndex = Index) then
         Result := Curr^.ObjectItem;
      Curr := Curr^.Next;
   end;
end;

procedure TDynamicList.Add(Value : TObject);
{ Procedure   : Add
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Adds a new node to the end of the linked list.  The counter is
                bumped up by 1.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Pointer used to create the new node }
   Nu : PointList;
begin
   { Create the node }
   New(Nu);
   { Set the items in the list.  Notice that the RecIndex will ALWAYS be 1 less
     than the count.  This is standard Delphi format. }
   Nu^.ObjectItem := Value;
   Nu^.Identifier := '';
   Nu^.RecIndex := FCount;
   Nu^.Next := Nil;
   { Set pointers }
   if (FTopItem = Nil) then
      FTopItem := Nu
   else
      FLastItem^.Next := Nu;
   FLastItem := Nu;
   { Finally, bump up the counter }
   FCount := FCount + 1;
end;

procedure TDynamicList.Clear;
{ Procedure   : Clear
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Attempts to free the attached objects and disposes of all the
                nodes.  It also resets the counter back to 0.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Pointer used to traverse the linked list }
   Curr : PointList;
begin
   { Loop until the linked list is destroyed }
   while(FTopItem <> Nil) do
   begin
      Curr := FTopItem^.Next;
      { Try to free the object, but for now don't crash the program if unable }
      try
         FTopItem^.ObjectItem.Free;
      except
      end;
      { Dispose of the node and setup for the next pass }
      Dispose(FTopItem);
      FTopItem := Curr;
   end;
   { Initialize variables }
   FTopItem := Nil;
   FLastItem := Nil;
   FCount := 0;
end;

constructor TDynamicLabel.Create(AOwner : TComponent);
{ Constructor : Create
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Creates the object that will be used to gather informaton.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   inherited Create(AOwner);
   FRemoteProcedure := '';
   FRemoteTag := '';
   FList := TDynamicList.Create;
   FEvents := TStringList.Create;
end;

destructor TDynamicLabel.Destroy;
{ Destructor  : Destroy
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Destroys the objects that it uses.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   FEvents.Free;
   FList.Free;
   inherited Destroy;
end;

procedure TDynamicLabel.DeActivate;
{ Procedure   : DeActivate
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Clears the list by freeing up the objects and then disposing
                the node in the linked list.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   FList.Clear;
end;

procedure TDynamicLabel.Activate;
{ Procedure   : Activate
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Activates this object.  The user MUST call this procedure at the
                create time of the form so that all the TLabels will be in place
                by the time the form shows up.  The user can specify their own
                RemoteProcedure and RemoteTag or leave them blank and accept the
                default.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { StringList of information }
   InfoList : TStringList;
begin
   if (RPCBroker <> Nil) then
   begin
      { Create the list that will have the TLabel information }
      InfoList := TStringList.Create;
      { Set up the call into the REMOTE PROCEDURE file (#8994) }
      if (FRemoteProcedure = '') then
         RPCBroker.RemoteProcedure := 'DYNAMIC TLABEL'
      else
         RPCBroker.RemoteProcedure := FRemoteProcedure;
      { The optional tag will tell label or routine to run }
      if (FRemoteTag = '') then
         RPCBroker.Param[0].Value := 'INIT'
      else
         RPCBroker.Param[0].Value := FRemoteTag;
      RPCBroker.Param[0].PType := literal;
      { Make the call }
      RPCBroker.lstCall(InfoList);
      { Process the information }
      if (InfoList.Count > 0) then
         MakeLabels(InfoList);
      InfoList.Free;
   end;
end;

procedure TDynamicLabel.MakeLabels(var MasterList : TStringList);
{ Procedure   : MakeLabels
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : This routine creates the labels given the information that was
                pulled down from the M Server.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Stringlist that contains a single labels information }
   SingleLabel : TStringList;
begin
   { Initialize }
   SingleLabel := TStringList.Create;
   { Take the master list, break it up into individual labels and create them }
   while(MasterList.Count > 0) do
   begin
      SeparateLabel(MasterList, SingleLabel);
      CreateLabel(SingleLabel);
   end;
   SingleLabel.Free;
end;

procedure TDynamicLabel.CreateLabel(SingleLabel : TStringList);
{ Procedure   : CreateLabel
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Reads the information in for one label and calls the appropriate
                routine which will assign the property properly.  When finished
                it assigns the identifier and record number in the list.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Index into the stringlist }
   I     : Integer;
   { TLabel object to create }
   Nu    : TLabel;
   { String used to determine what to load }
   Value : String;
begin
   { Initialize.  Create a TLabel and set the owner and parent properties.  These
     are the only properties that are required to be set in order to display a
     label.  If nothing else is set, then only a blank label is displayed. }
   Nu := TLabel.Create(Owner);
   Nu.Parent := TWinControl(Owner);
   SetName(Nu,SingleLabel[0]);
   SetCaption(Nu,'');
   { Loop through the information and set the properties and events }
   for I := 0 to (SingleLabel.Count - 1) do
   begin
      Value := UpperCase(Piece(SingleLabel[I],'^',2));
      if (Value = 'ALIGN') then
         SetAlign(Nu,SingleLabel[I]);
      if (Value = 'ALIGNMENT') then
         SetAlignment(Nu,SingleLabel[I]);
      if (Value = 'AUTOSIZE') then
         SetAutoSize(Nu,SingleLabel[I]);
      if (Value = 'CAPTION') then
         SetCaption(Nu,SingleLabel[I]);
      if (Value = 'COLOR') then
         SetColor(Nu,SingleLabel[I]);
      if (Value = 'CURSOR') then
         SetCursor(Nu,SingleLabel[I]);
      if (Value = 'DRAGCURSOR') then
         SetDragCursor(Nu,SingleLabel[I]);
      if (Value = 'DRAGMODE') then
         SetDragMode(Nu,SingleLabel[I]);
      if (Value = 'ENABLED') then
         SetEnabled(Nu,SingleLabel[I]);
      if (Value = 'FONT|COLOR') then
         SetFontColor(Nu,SingleLabel[I]);
      if (Value = 'HEIGHT') then
         SetHeight(Nu,SingleLabel[I]);
      if (Value = 'LEFT') then
         SetLeft(Nu,SingleLabel[I]);
      if (Value = 'PARENTCOLOR') then
         SetParentColor(Nu,SingleLabel[I]);
      if (Value = 'PARENTFONT') then
         SetParentFont(Nu,SingleLabel[I]);
      if (Value = 'PARENTSHOWHINT') then
         SetParentShowHint(Nu,SingleLabel[I]);
      if (Value = 'SHOWACCELCHAR') then
         SetShowAccelChar(Nu,SingleLabel[I]);
      if (Value = 'SHOWHINT') then
         SetShowHint(Nu,SingleLabel[I]);
      if (Value = 'TAG') then
         SetTag(Nu,SingleLabel[I]);
      if (Value = 'TOP') then
         SetTop(Nu,SingleLabel[I]);
      if (Value = 'TRANSPARENT') then
         SetTransparent(Nu,SingleLabel[I]);
      if (Value = 'VISIBLE') then
         SetVisible(Nu,SingleLabel[I]);
      if (Value = 'WIDTH') then
         SetWidth(Nu,SingleLabel[I]);
      if (Value = 'WORDWRAP') then
         SetWordWrap(Nu,SingleLabel[I]);
      if (Value = 'ONCLICK') then
         SetOnClick(Nu,SingleLabel[I]);
      if (Value = 'ONDBLCLICK') then
         SetOnDblClick(Nu,SingleLabel[I]);
      if (Value = 'ONDRAGDROP') then
         SetOnDragDrop(Nu,SingleLabel[I]);
      if (Value = 'ONDRAGOVER') then
         SetOnDragOver(Nu,SingleLabel[I]);
      if (Value = 'ONENDDRAG') then
         SetOnEndDrag(Nu,SingleLabel[I]);
      if (Value = 'ONMOUSEDOWN') then
         SetOnMouseDown(Nu,SingleLabel[I]);
      if (Value = 'ONMOUSEMOVE') then
         SetOnMouseMove(Nu,SingleLabel[I]);
      if (Value = 'ONMOUSEUP') then
         SetOnMouseUp(Nu,SingleLabel[I]);
      if (Value = 'ONSTARTDRAG') then
         SetOnStartDrag(Nu,SingleLabel[I]);
   end;
   { Add to the list and set the identifier }
   FList.Add(Nu);
   FList.SetIdentifier(Piece(SingleLabel[0],'^',1),(FList.Count-1));
end;

procedure TDynamicLabel.ChangeParam(SingleLabel : TStringList; ListIndex : Integer);
{ Procedure   : ChangeParam
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Reads information in and calls the appropriate procedure to change
                the information.  This is normally done when an event of some sort
                has happened.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Index into the stringlist }
   I     : Integer;
   { Label to Change }
   Nu    : TLabel;
   { String used to determine what to load }
   Value : String;
begin
   { Get the object in question given the index in the list }
   Nu := TLabel(FList.Item[ListIndex]);
   { Loop through changing the properties until done }
   for I := 0 to (SingleLabel.Count - 1) do
   begin
      Value := UpperCase(Piece(SingleLabel[I],'^',2));
      if (Value = 'ALIGN') then
         SetAlign(Nu,SingleLabel[I]);
      if (Value = 'ALIGNMENT') then
         SetAlignment(Nu,SingleLabel[I]);
      if (Value = 'AUTOSIZE') then
         SetAutoSize(Nu,SingleLabel[I]);
      if (Value = 'CAPTION') then
         SetCaption(Nu,SingleLabel[I]);
      if (Value = 'COLOR') then
         SetColor(Nu,SingleLabel[I]);
      if (Value = 'CURSOR') then
         SetCursor(Nu,SingleLabel[I]);
      if (Value = 'DRAGCURSOR') then
         SetDragCursor(Nu,SingleLabel[I]);
      if (Value = 'DRAGMODE') then
         SetDragMode(Nu,SingleLabel[I]);
      if (Value = 'ENABLED') then
         SetEnabled(Nu,SingleLabel[I]);
      if (Value = 'FONT|COLOR') then
         SetFontColor(Nu,SingleLabel[I]);
      if (Value = 'HEIGHT') then
         SetHeight(Nu,SingleLabel[I]);
      if (Value = 'LEFT') then
         SetLeft(Nu,SingleLabel[I]);
      if (Value = 'PARENTCOLOR') then
         SetParentColor(Nu,SingleLabel[I]);
      if (Value = 'PARENTFONT') then
         SetParentFont(Nu,SingleLabel[I]);
      if (Value = 'PARENTSHOWHINT') then
         SetParentShowHint(Nu,SingleLabel[I]);
      if (Value = 'SHOWACCELCHAR') then
         SetShowAccelChar(Nu,SingleLabel[I]);
      if (Value = 'SHOWHINT') then
         SetShowHint(Nu,SingleLabel[I]);
      if (Value = 'TAG') then
         SetTag(Nu,SingleLabel[I]);
      if (Value = 'TOP') then
         SetTop(Nu,SingleLabel[I]);
      if (Value = 'TRANSPARENT') then
         SetTransparent(Nu,SingleLabel[I]);
      if (Value = 'VISIBLE') then
         SetVisible(Nu,SingleLabel[I]);
      if (Value = 'WIDTH') then
         SetWidth(Nu,SingleLabel[I]);
      if (Value = 'WORDWRAP') then
         SetWordWrap(Nu,SingleLabel[I]);
      if (Value = 'ONCLICK') then
         SetOnClick(Nu,SingleLabel[I]);
      if (Value = 'ONDBLCLICK') then
         SetOnDblClick(Nu,SingleLabel[I]);
      if (Value = 'ONDRAGDROP') then
         SetOnDragDrop(Nu,SingleLabel[I]);
      if (Value = 'ONDRAGOVER') then
         SetOnDragOver(Nu,SingleLabel[I]);
      if (Value = 'ONENDDRAG') then
         SetOnEndDrag(Nu,SingleLabel[I]);
      if (Value = 'ONMOUSEDOWN') then
         SetOnMouseDown(Nu,SingleLabel[I]);
      if (Value = 'ONMOUSEMOVE') then
         SetOnMouseMove(Nu,SingleLabel[I]);
      if (Value = 'ONMOUSEUP') then
         SetOnMouseUp(Nu,SingleLabel[I]);
      if (Value = 'ONSTARTDRAG') then
         SetOnStartDrag(Nu,SingleLabel[I]);
   end;
end;

procedure TDynamicLabel.SetAlign(Nu : TLabel; Info : String);
{ Procedure   : SetAlign
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the align property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Align := TAlign(StrToInt(Piece(Info,'^',3)));
end;

procedure TDynamicLabel.SetAlignment(Nu : TLabel; Info : String);
{ Procedure   : SetAlignment
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the alignment property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.AlignMent := TAlignment(StrToInt(Piece(Info,'^',3)));
end;

procedure TDynamicLabel.SetAutoSize(Nu : TLabel; Info : String);
{ Procedure   : SetAutoSize
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the autosize property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.AutoSize := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetCaption(Nu : TLabel; Info : String);
{ Procedure   : SetCaption
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the caption property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Caption := Piece(Info,'^',3);
end;

procedure TDynamicLabel.SetColor(Nu : TLabel; Info : String);
{ Procedure   : SetColor
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the color property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Color := TColor(StrToInt(Piece(Info,'^',3)));
end;

procedure TDynamicLabel.SetCursor(Nu : TLabel; Info : String);
{ Procedure   : SetCursor
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Cursor property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetDragCursor(Nu : TLabel; Info : String);
{ Procedure   : SetDragCursor
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the DragCursor property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetDragMode(Nu : TLabel; Info : String);
{ Procedure   : SetDragMode
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the DragMode property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetEnabled(Nu : TLabel; Info : String);
{ Procedure   : SetEnabled
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Enabled property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Enabled := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetFontColor(Nu : TLabel; Info : String);
{ Procedure   : SetFontColor
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Color portion of the Font property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Font.Color := TColor(StrToInt(Piece(Info,'^',3)));
end;

procedure TDynamicLabel.SetHeight(Nu : TLabel; Info : String);
{ Procedure   : SetHeight
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Height property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Height := StrToInt(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetLeft(Nu : TLabel; Info : String);
{ Procedure   : SetLeft
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Left property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Left := StrToInt(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetName(Nu : TLabel; Info : String);
{ Procedure   : SetName
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Name property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Name := Piece(Info,'^',1);
end;

procedure TDynamicLabel.SetParentColor(Nu : TLabel; Info : String);
{ Procedure   : SetParentColor
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the ParentColor property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.ParentColor := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetParentFont(Nu : TLabel; Info : String);
{ Procedure   : SetParentFont
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the ParentFont property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.ParentFont := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetParentShowHint(Nu : TLabel; Info : String);
{ Procedure   : SetParentShowHint
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the ParentShowHint property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.ParentShowHint := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetShowAccelChar(Nu : TLabel; Info : String);
{ Procedure   : SetShowAccelChar
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the ShowAccelChar property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.ShowAccelChar := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetShowHint(Nu : TLabel; Info : String);
{ Procedure   : SetShowHint
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the ShowHint property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.ShowHint := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetTag(Nu : TLabel; Info : String);
{ Procedure   : SetTag
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Tag property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Tag := StrToInt(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetTop(Nu : TLabel; Info : String);
{ Procedure   : SetTop
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Top property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Top := StrToInt(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetTransparent(Nu : TLabel; Info : String);
{ Procedure   : SetTransparent
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Transparent property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Transparent := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetVisible(Nu : TLabel; Info : String);
{ Procedure   : SetVisible
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Visible property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Visible := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetWidth(Nu : TLabel; Info : String);
{ Procedure   : SetWidth
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the Width property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.Width := StrToInt(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetWordWrap(Nu : TLabel; Info : String);
{ Procedure   : SetWordWrap
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Sets the WordWrap property.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   Nu.WordWrap := StrToBool(Piece(Info,'^',3));
end;

procedure TDynamicLabel.SetOnClick(Nu : TLabel; Info : String);
{ Procedure   : SetOnClick
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnClick property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   FEvents.Add(Info);
   Nu.OnClick := Click;
end;

procedure TDynamicLabel.SetOnDblClick(Nu : TLabel; Info : String);
{ Procedure   : SetOnDblClick
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnDblClick property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetOnDragDrop(Nu : TLabel; Info : String);
{ Procedure   : SetOnDragDrop
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnDragDrop property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetOnDragOver(Nu : TLabel; Info : String);
{ Procedure   : SetOnDragOver
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnDragOver property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetOnEndDrag(Nu : TLabel; Info : String);
{ Procedure   : SetOnEndDrag
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnEndDrag property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetOnMouseDown(Nu : TLabel; Info : String);
{ Procedure   : SetOnMouseDown
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnMouseDown property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetOnMouseMove(Nu : TLabel; Info : String);
{ Procedure   : SetOnMouseMove
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnMouseMove property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetOnMouseUp(Nu : TLabel; Info : String);
{ Procedure   : SetOnMouseUp
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnMouseUp property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.SetOnStartDrag(Nu : TLabel; Info : String);
{ Procedure   : SetOnStartDrag
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Assigns the OnStartDrag property to point to an event that calls
                a procedure on the M Side.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
end;

procedure TDynamicLabel.Click(Sender : TObject);
{ Procedure   : Click
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : This is a TNotifyEvent that the OnClick property in each of the
                TLabels points to.  This procedure will get the RPCBroker ready
                and then send to the M side information that can be used to handle
                the event.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Information passed back from the M side }
   InfoList  : TStringList;
   { Index of the TLabel in the list }
   ListIndex : Integer;
begin
   with Sender as TLabel do
   begin
      { Initialize }
      InfoList := TStringList.Create;
      { Get the index }
      ListIndex := FList.Find(Name);
      { Load the RPCBroker for this event }
      LoadEvent('ONCLICK',TLabel(Sender));
      { If information is loaded ... }
      if (RPCBroker.Param[0].Value <> '') then
      begin
         { Make that call }
         RPCBroker.lstCall(InfoList);
         { Change parameters }
         ChangeParam(InfoList,ListIndex);
      end;
      InfoList.Free;
   end;
end;

procedure TDynamicLabel.DblClick(Sender : TObject);
begin
end;

procedure TDynamicLabel.DragDrop(Sender,Source : TObject; X,Y : Integer);
begin
end;

procedure TDynamicLabel.DragOver(Sender,Source : TObject; X,Y : Integer;
                                 State : TDragState; var Accept : Boolean);
begin
end;

procedure TDynamicLabel.EndDrag(Sender,Target : TObject; X,Y : Integer);
begin
end;

procedure TDynamicLabel.MouseDown(Sender : TObject; Button : TMouseButton;
                                  Shift : TShiftState; X,Y : Integer);
begin
end;

procedure TDynamicLabel.MouseMove(Sender : TObject; Shift : TShiftState;
                                  X,Y : Integer);
begin
end;

procedure TDynamicLabel.MouseUp(Sender : TObject; Button : TMouseButton;
                                Shift : TShiftState; X,Y : Integer);
begin
end;

procedure TDynamicLabel.StartDrag(Sender : TObject; var DragObject : TDragObject);
begin
end;

procedure TDynamicLabel.LoadEvent(CurrEvent : String; Curr : TLabel);
{ Procedure   : LoadEvent
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Loads information into the RPCBroker in preparation for movement.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Index for FEvents }
   I         : Integer;
   { Quit when found }
   Found     : Boolean;
   { Name of the label }
   Name      : String;
begin
   { Initialize }
   Found := False;
   I := 0;
   Name := UpperCase(Curr.Name);
   RPCBroker.Param[0].Value := '';
   { Setup the remote procedure call.  The default is DYNAMIC TLABEL which can
     be overriden. }
   if (FRemoteProcedure = '') then
      RPCBroker.RemoteProcedure := 'DYNAMIC TLABEL'
   else
      RPCBroker.RemoteProcedure := FRemoteProcedure;
   { Find the event for this TLabel and load it in the first parameter }
   while((Not Found) and (I < FEvents.Count)) do
   begin
      if ((Name = UpperCase(Piece(FEvents[I],'^',1))) and
          (CurrEvent = UpperCase(Piece(FEvents[I],'^',2)))) then
      begin
         Found := True;
         { Load the indirection that will handle the event }
         RPCBroker.Param[0].Value := UpperCase(Piece(FEvents[I],'^',3));
         if (Piece(FEvents[I],'^',4) <> '') then
            RPCBroker.Param[0].Value := RPCBroker.Param[0].Value + '^' +
               UpperCase(Piece(FEvents[I],'^',4));
         RPCBroker.Param[0].PType := literal;
      end;
      I := I + 1;
   end;
   { If the particular event was found, then finish loading the properties of
     the TLabel. }
   if (Found) then
      LoadParams(Curr);
end;

procedure TDynamicLabel.LoadParams(Curr : TLabel);
{ Procedure   : LoadParams
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Load all the properties of the TLabel so that they will be
                available for the event.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial.  Only loaded the
                background color at this time for testing purposes.
}
var
   { Name of the label }
   Name : String;
begin
   Name := UpperCase(Curr.Name);
   RPCBroker.Param[1].Mult['0'] := Name+'^ALIGN^'+IntToStr(Ord(Curr.Align));
   RPCBroker.Param[1].Mult['1'] := Name+'^ALIGNMENT^'+IntToStr(Ord(Curr.Alignment));
   RPCBroker.Param[1].Mult['2'] := Name+'^AUTOSIZE^'+BoolToStr(Curr.AutoSize);
   RPCBroker.Param[1].Mult['3'] := Name+'^CAPTION^'+Curr.Caption;
   RPCBroker.Param[1].Mult['4'] := Name+'^COLOR^'+IntToStr(Ord(Curr.Color));
   RPCBroker.Param[1].Mult['5'] := Name+'^CURSOR^'+IntToStr(Ord(Curr.Cursor));
   RPCBroker.Param[1].Mult['6'] := Name+'^DRAGCURSOR^'+IntToStr(Ord(Curr.Cursor));
   RPCBroker.Param[1].Mult['7'] := Name+'^DRAGMODE^'+IntToStr(Ord(Curr.DragMode));
   RPCBroker.Param[1].Mult['8'] := Name+'^ENABLED^'+BoolToStr(Curr.Enabled);
   RPCBroker.Param[1].Mult['9'] := Name+'^FONT|COLOR^'+IntToStr(Ord(Curr.Font.Color));
   RPCBroker.Param[1].Mult['10'] := Name+'^HEIGHT^'+IntToStr(Curr.Height);
   RPCBroker.Param[1].Mult['11'] := Name+'^LEFT^'+IntToStr(Curr.Left);
   RPCBroker.Param[1].Mult['12'] := Name+'^PARENTCOLOR^'+BoolToStr(Curr.ParentColor);
   RPCBroker.Param[1].Mult['13'] := Name+'^PARENTFONT^'+BoolToStr(Curr.ParentFont);
   RPCBroker.Param[1].Mult['14'] := Name+'^PARENTSHOWHINT^'+BoolToStr(Curr.ParentShowHint);
   RPCBroker.Param[1].Mult['15'] := Name+'^SHOWACCELCHAR^'+BoolToStr(Curr.ShowAccelChar);
   RPCBroker.Param[1].Mult['16'] := Name+'^SHOWHINT^'+BoolToStr(Curr.ShowHint);
   RPCBroker.Param[1].Mult['17'] := Name+'^TAG^'+IntToStr(Curr.Tag);
   RPCBroker.Param[1].Mult['18'] := Name+'^TOP^'+IntToStr(Curr.Top);
   RPCBroker.Param[1].Mult['19'] := Name+'^TRANSPARENT^'+BoolToStr(Curr.Transparent);
   RPCBroker.Param[1].Mult['20'] := Name+'^VISIBLE^'+BoolToStr(Curr.Visible);
   RPCBroker.Param[1].Mult['21'] := Name+'^WIDTH^'+IntToStr(Curr.Width);
   RPCBroker.Param[1].Mult['22'] := Name+'^WORDWRAP^'+BoolToStr(Curr.WordWrap);
   RPCBroker.Param[1].PType := list;
end;

procedure TDynamicLabel.SeparateLabel(var MasterList, SingleList : TStringList);
{ Procedure   : SeparateLabel
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : This is a destructive procedure in that the MasterList is destroyed
                in order to build the parsed out singlelist.  Basically, the name is
                taken from the 0 node and all pieces that have that node are put in
                the single list.  At the same time all pieces that have that node are
                removed from the MasterList.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
var
   { Index counter into the MasterList }
   I    : Integer;
   { Name of the label }
   Name : String;
   { String used to help traverse the list picking out only the current label }
   Temp : String;
begin
   { Initialize.  Ensure that the singlelist is empty. }
   SingleList.Clear;
   { If the MasterList is NOT empty, then get the name of the label to parse out. }
   if (MasterList.Count > 0) then
      Name := Piece(MasterList[0],'^',1);
   I := 0;
   { Loop until the done with the MasterList }
   while(I < MasterList.Count) do
   begin
      { Get the name found on the current node. }
      Temp := Piece(MasterList[I],'^',1);
      { If it matches, then pull it out. }
      if (Temp = Name) then
      begin
         SingleList.Add(MasterList[I]);
         MasterList.Delete(I);
      end
      { Only increment if the names did not match (Very important). }
      else
         I := I + 1;
   end;
end;

Function TDynamicLabel.BoolToStr(Value : Boolean) : String;
{ Function    : BoolToStr
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Since I prefer to send a string of TRUE for True and FALSE for
                False (as opposed to 0 and 1), I wrote this little routine which
                will do it for me.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   if(Value) then
      Result := 'TRUE'
   else
      Result := 'FALSE';
end;

Function TDynamicLabel.StrToBool(Value : String) : Boolean;
{ Function    : StrToBool
  Written By  : Michael S. Simpson
  Date        : 7 April 1998
  Purpose     : Since I prefer to convert a string of TRUE for True and FALSE for
                False (as opposed to 0 and 1), I wrote this little routine which
                will do it for me.
  Modified By : Michael S. Simpson - 7 April 1998 - Initial
}
begin
   if(UpperCase(Value) = 'TRUE') then
      Result := True
   else
      Result := False;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TDynamicLabel]);
end;

end.
